Package: xpose.nlmixr2
Type: Package
Title: Graphical Diagnostics for Pharmacometric Models: Extension to
        'nlmixr2'
Version: 0.4.2
Authors@R: c(person('Justin', 'Wilkins', email = 'justin.wilkins@occams.com',
                  role = c('aut', 'cre', 'cph')),
          person('Matthew','Fidler',
      role = c('aut','cph'), email = 'matthew.fidler@gmail.com'),
            person('Benjamin', 'Guiastrennec',
             role = c('aut'),
             email = 'guiastrennec@gmail.com'),
      person('Andrew C.', 'Hooker',
             role = c('aut'),
             email = 'andrew.hooker@farmbio.uu.se'),
      person('Anna', 'Olofsson', role = c('aut', 'cph')),
      person('Sebastian', 'Ueckert',
             role = c('aut'),
             email = 'sebastian.ueckert@farmbio.uu.se'),
      person('Ron', 'Keizer',
             role = 'aut',
             email = 'ronkeizer@gmail.com'),
      person('Kajsa', 'Harling', role = 'ctb'),
      person('Mike K.', 'Smith', role = 'ctb'),
      person('Elodie', 'Plan', role = 'ctb'),
      person('Mats O.', 'Karlsson',
             role = c('aut', 'cph'),
             email = 'mats.karlsson@farmbio.uu.se'),
      person('Pharmetheus', role = 'ctb'),
      person('Pfizer', role = 'ctb'),
      person('Occams', role = 'ctb'),
      person('Novartis', role = 'ctb'))
Maintainer: Justin Wilkins <justin.wilkins@occams.com>
Description: Extension to 'xpose' to support 'nlmixr2'. Provides functions to import 'nlmixr2' fit data into an 'xpose' data object, allowing the use of 'xpose' for 'nlmixr2' model diagnostics.
License: GPL (>= 2)
Encoding: UTF-8
Depends: R (>= 3.2), xpose (>= 0.4.2)
Imports: ggplot2 (>= 2.2.1), dplyr (>= 0.7.4), tibble (>= 2.0.0),
        stringr (>= 1.2.0), tidyr (>= 0.7.2), magrittr (>= 1.5),
        methods (>= 3.4.1), vpc (>= 1.0.2), crayon, rlang, nlmixr2est
Suggests: readr, nlmixr2, nlmixr2data, testthat (>= 3.0.0)
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-09 06:38:45 UTC; justin
Author: Justin Wilkins [aut, cre, cph],
  Matthew Fidler [aut, cph],
  Benjamin Guiastrennec [aut],
  Andrew C. Hooker [aut],
  Anna Olofsson [aut, cph],
  Sebastian Ueckert [aut],
  Ron Keizer [aut],
  Kajsa Harling [ctb],
  Mike K. Smith [ctb],
  Elodie Plan [ctb],
  Mats O. Karlsson [aut, cph],
  Pharmetheus [ctb],
  Pfizer [ctb],
  Occams [ctb],
  Novartis [ctb]
Repository: CRAN
Date/Publication: 2026-05-09 14:40:02 UTC
Built: R 4.6.0; ; 2026-05-09 18:57:49 UTC; unix
