# xpose.nlmixr2

<!-- badges: start -->
[![CRAN checks](https://cranchecks.info/badges/summary/xpose.nlmixr2)](https://cran.r-project.org/web/checks/check_results_xpose.nlmixr2.html)
[![CRAN total downloads](https://cranlogs.r-pkg.org/badges/grand-total/xpose.nlmixr2)](https://cran.r-project.org/package=xpose.nlmixr2)
<!-- badges: end -->

The goal of `xpose.nlmixr2` is to provide an extension to `xpose` to
support `nlmixr2`. The package provides functions to import `nlmixr2`
fit data into an `xpose` data object, allowing the use of `xpose` for
`nlmixr2` model diagnostics.

The excellent `xpose` package can be found on [CRAN](https://cran.r-project.org/package=xpose) or on 
[GitHub](https://github.com/UUPharmacometrics/xpose), and is a
prerequisite for using `xpose.nlmixr2`.

## Installation

You can install the released version of `xpose.nlmixr2` from [CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("xpose.nlmixr2")
```

Alternatively, the development version can be installed from [GitHub](https://github.com/nlmixr2/xpose.nlmixr2):

``` r
devtools::install_github("nlmixr2/xpose.nlmixr2")
```


